#!/bin/csh

#
#  This script check whether all requred PDS files are present on the 
#  master image volume (the one with all UNIX files with CRs added.)
#
#  This is a super-duper-light version of the PDS's VVTOOL :-). So,
#  don't expect much from it ...
#  LSE/NAIF, June 20, 2002
#
#     Made modifications, including lower case, for PDS-D. NOTE: Unlike 
#     other scripts, we had to make a (UPPER/lower) case choice.
#     This script will not work with the old PDS files.
#  
#  BVS/NAIF, April 25, 2000.
#
#     Added second pass capturing all text files (*.TXT, *.LBL, *.CAT ...)
#
#  BVS/NAIF, January 10, 2000.
#
#     Updated line length check to be for 78 char.
#
#  BVS/NAIF, January 5, 2000.
#

#
#  badchar executable
#
set BADCHAR = '/home/bsemenov/bin/badchar'

echo " "
echo "Problems are marked with (E) in at the beginnng of report lines."
echo " "
#
#  Checking directories.
#
echo "   Verifying directories:"
echo " "

foreach DD ( catalog data data/ck data/ek data/fk data/ik data/lsk data/pck data/sclk data/spk document index software )
   
   #
   #  Does it exist?
   #
   if ( -e $DD ) then
   
      #
      #  Is it a directory?
      #
      if ( -d $DD ) then
         echo "      $DD exists and is a directory."
      else
         echo "(E)   $DD is not a directory."
      endif
      
   else
      echo "(E)   $DD doesn't exist."     
   endif

end

#
#  Checking files.
#
echo " "
echo "   Verifying files -- first pass is over REQUIRED files:"
echo " "

foreach FF ( aareadme.htm aareadme.lbl aareadme.txt catalog/catinfo.txt catalog/insthost.cat catalog/mission.cat catalog/person.cat catalog/ref.cat catalog/spiceds.cat data/ck/ckinfo.txt data/ek/ekinfo.txt data/fk/fkinfo.txt data/ik/ikinfo.txt data/lsk/lskinfo.txt data/pck/pckinfo.txt data/sclk/sclkinfo.txt data/spk/spkinfo.txt document/docinfo.txt document/lblinfo.txt document/onlabels.txt index/index.lbl index/index.tab index/indxinfo.txt software/softinfo.txt voldesc.cat )

   #
   #  Does it exist?
   #
   if ( -e $FF ) then
   
      #
      #  Does it have non-zero size?
      #
      if ( ! ( -z $FF ) ) then
      
         #
         #  Does it have any bad characters?
         #
         $BADCHAR -s $FF
         
         if ( $status != 1 ) then
            
            #
            #  Does it have lines longer than 78 characters?
            #
            awk -e '{if(length>78){print $0}}' < $FF | grep '.' > /dev/null
            
            if ( ( $status != 0 ) || $FF == "aareadme.htm" || $FF == "index/index.tab" ) then
               
               echo "      $FF is OK."
               
            else
               #
               #  Yes it does. Run awk again to dump these lines.
               #
               echo "(E)   $FF has lines longer than 78 char. They are:"
               echo " "
               awk -e '{if(length>78){print $0}}' < $FF
               echo " "
            endif
            
         else
            #
            #  Yes, it does have bad characters. Run badchar again
            #  to generate report.
            #
            echo "(E)   $FF has bad characaters. badchar report is:"
            echo " "
            $BADCHAR -v $FF
            echo " "
         endif

      else
         echo "(E)   $FF has zero size."
      endif
      
   else
      echo "(E)   $FF doesn't exist."     
   endif

end

#
#  Checking files.
#
echo " "
echo "   Verifying files -- second pass is over all text files:"
echo " "

foreach FF ( `find . -name '*.lbl' -print` `find . -name '*.txt' -print` `find . -name '*.cat' -print` `find . -name '*.htm' -print` `find . -name '*.tab' -print`)

   #
   #  Does it exist?
   #
   if ( -e $FF ) then
   
      #
      #  Does it have non-zero size?
      #
      if ( ! ( -z $FF ) ) then
      
         #
         #  Does it have any bad characters?
         #
         $BADCHAR -s $FF
         
         if ( $status != 1 ) then
            
            #
            #  Does it have lines longer than 78 characters?
            #
            awk -e '{if(length>78){print $0}}' < $FF | grep '.' > /dev/null
            
            if ( ( $status != 0 ) || $FF:e == "htm" || $FF:e == "tab" ) then
               
               echo "      $FF is OK."
               
            else
               #
               #  Yes it does. Run awk again to dump these lines.
               #
               echo "(E)   $FF has lines longer than 78 char. They are:"
               echo " "
               awk -e '{if(length>78){print $0}}' < $FF
               echo " "
            endif
            
         else
            #
            #  Yes, it does have bad characters. Run badchar again
            #  to generate report.
            #
            echo "(E)   $FF has bad characaters. badchar report is:"
            echo " "
            $BADCHAR -v $FF
            echo " "
         endif

      else
         echo "(E)   $FF has zero size."
      endif
      
   else
      echo "(E)   $FF doesn't exist."     
   endif

end
echo " "
echo "All done."
echo " "

exit
