#!/usr/local/bin/perl
#
#
#  2.0.2  May 18, 2004 LSE/NAIF
#
#  Further modified so that string manipulation does not depend on 
#  any character string: the directory is determined by stripping
#  everything after the /.
#  
#  to kernel.list. Keeps track of file names to eliminate duplicates.
#  2.0.1  Oct 23, 2002 LSE/NAIF
#
#  Modified so that string manipulation does not depend on Odyssey (e.g. 'odsp_..)
#  Also, allow for no directory name and output list file name change
#  to kernel.list. Keeps track of file names to eliminate duplicates.
#
#  2.0.0, Oct 3, 2002, LSE/NAIF
#
#  Modified so that common header tokens are **not** removed. This is to 
#  accomodate new tokens RELEASE_ID and RELEASE_DATE that are used by 
#  make_index.pl.
#
#
#  1.0.0, June 3, 2002, LSE/NAIF
#  This script reads a file list input file from the command line (usually called
#  concat_input_list), containing names (e.g. data/ck/odsp_1000_ck_list) 
#  of input files in each kernel directory (note that file name for individual lists
#  must start with "odsp"). It takes the common (header) tokens (e.g.
#  DATE, SPACECRAFT, etc) from the first odsp*_list file and adds the
#  FILE, MAKLABEL_OPTIONS and DESCRIPTION tokens from the first and subsequent
#  tokens (FILE, MAKLABEL_OPTIONS and DESCRIPTION) from the following
#  *_list files. It adds the pathname (e.g. data/ck) to each FILE value.
#  The result is a super (concatenated) file input file
#  (usually called odsp_1000.kernel_list) containing tokens from all the 
#  directories combined. This script is meant to be run from the top level 
#  volume file for Odyssey data.
#
#  Usage:
#  concat.pl concat_input_list
#
#
$scriptname  = "concat.pl";
$version     = "2.0.2  May 18, 2004 LSE/NAIF";
$kernel_list = "kernel_list";

#
#
print "\nScript that creates a text file which lists a complete collection ".
      "\n of SPICE kernels.\n\n".
      "Version $version.\n";

#
#  Get from the command line the name of the file containing list
#  of files containing labeled kernels and other meta-keywords.
#
if ( @ARGV == 1 ) {
   $filelist = shift @ARGV;
   if ( ! -e $filelist ) {
      die "\nERROR:$scriptname: the file '$filelist' doesn't exist.\n\n";
   }
} else {
   die "\nUsage: > $scriptname <kernel_list_file_name>\n\n";
}


#
#  Load list of kernel file lists.
#

print "\nReading file name lists. ...\n";

#  Open file list input file.
open( LIST, "< $filelist" ) || 
   die "\nERROR:$scriptname: cannot open '$filelist'.\n\n";

#  Open file for concatenated output.
open( KLIST , "> $kernel_list" ) || 
   die "\nERROR:$scriptname: cannot open '$kernel_list'.\n\n";
  
while ( $listfile = <LIST> ) {

  chop( $listfile ) ;

    if  ( ! ( $listfile =~ /^#/ ) ) {

#  First find directory path (if present). (Note "m" denotes special delimiter)
     if ( $listfile =~ m#/# ) {

        ( $dirname = $listfile ) =~ s#/[^/]*$## ;

     }else {

       $dirname ="" ;

     }
	print "Directory name $dirname \n";

#   Open list file in input. 

      open( LISTF , "< $listfile" ) ||
       die "\nERROR:$scriptname: cannot open '$listfile'.\n\n";

    print "\nReading file name '$listfile'.\n\n";

#  Set a flag for header read.

      while ( $listline = <LISTF> ) {

         chop( $listline );
    
#  Unset filename marker array with dummy filename. This is so lines
#will be written to output.
		$filename    = "placeholder" ;
		$a_filename{$filename} = "" ;

#check for "FILE" token

	    if ( $listline =~ /^FILE/ )  {


# It's a file. Store name in an array to check for duplicates.
# First store last filename in marker array a_filename.
		($filename = $listline ) =~s/^FILE\s*=\s*// ;

		if ( $dirname ) {
			$listline =~ s#= #= $dirname/# ;
		}

	    }

# Now write out only if not already there.

	    if ( ! ($a_filename{$filename} ) ) {
              print KLIST "$listline\n" ;
	      if ($filename ne "placeholder") {
	   	   $a_filename{$filename} = $filename ;
	      }
            }

      }
         close( LISTF ) || 
          die "\nERROR:$scriptname: cannot close '$listfile'.\n\n"; 

    }

}

close( KLIST ) ||  
   die "\nERROR:$scriptname: cannot close '$kernel_list'.\n\n"; 

close( LIST ) ||  
   die "\nERROR:$scriptname: cannot close '$filelist'.\n\n"; 

print "done.\n";
